
import cv2
from PIL import Image
from unihiker import GUI
from pinpong.board import Board,Pin
from pinpong.extension.unihiker import *
import numpy as np
from pinpong.board import  Servo # 导入Servo库

mid = 82
angle = mid
stop_h = 400  #停止距离的上限
stop_l = 380  #停止距离的下限


a = [175,98,67,54,42,37]  #x 代表脸的宽度
b = [100,200,300,400,500,600] #y 代表手和摄像机之间的距离(mm)
# 因此我们需要一个类似 y = AX^2 + BX + C 的方程来拟合

coff = np.polyfit(a, b, 2)  #构造二阶多项式方程
A, B, C = coff

# 载入预训练的人脸检测模型，这里使用的是Haar特征分类器
face_cascade = cv2.CascadeClassifier(cv2.data.haarcascades + 'haarcascade_frontalface_default.xml')

u_gui=GUI()
Board().begin()
#打开摄像头
vd = cv2.VideoCapture(0)
# 设置视频捕获属性
vd.set(cv2.CAP_PROP_FRAME_WIDTH, 320)
vd.set(cv2.CAP_PROP_FRAME_HEIGHT, 240)
vd.set(cv2.CAP_PROP_BUFFERSIZE, 1)

# 检查设置是否成功
frame_width = vd.get(cv2.CAP_PROP_FRAME_WIDTH)
frame_height = vd.get(cv2.CAP_PROP_FRAME_HEIGHT)
buffer_size = vd.get(cv2.CAP_PROP_BUFFERSIZE)

print(f"Frame Width: {frame_width}, Frame Height: {frame_height}, Buffer Size: {buffer_size}")

# 示例参数
width, height = frame_width, frame_height  # 摄像头的视场宽度和高度
center_radius = 0  # 中心圈半径

bgImg=u_gui.draw_image(image="",x=0,y=0)
ltTxt=u_gui.draw_text(text="行空板",x=10,y=10,font_size=28, color="#00FF00")
ltTxt1=u_gui.draw_text(text="实际距离",x=10,y=250,font_size=10, color="#00FFFF")
ltTxt2=u_gui.draw_text(text="人脸宽度",x=10,y=280,font_size=10, color="#00FFFF")

#=======================================================================================
s1 = Servo(Pin(Pin.D0)) # 将Pin传入Servo中初始化舵机引脚

s1.write_angle(mid)  # 控制舵机转到90度位置 方法2
time.sleep(0.5)

p_1 = Pin(Pin.P1, Pin.OUT) #引脚初始化为电平输出
p_2 = Pin(Pin.P2, Pin.OUT) #引脚初始化为电平输出
p_1.value(1)
p_2.value(0)

# PWM模拟输出引脚支持: P0  P2  P3  P10  P16  P21  P22  P23
#pwm3 = PWM(Pin(Pin.P3)) #将引脚传入PWM初始化  模拟输出方法1
pwm9 = Pin(Pin.P9, Pin.PWM) #初始化引脚为PWM模式 模拟输出方法2

def car(speed, direction):
    # 控制舵机角度
    # 将方向值映射到30-140度
    angle = mid + direction  # 从85度开始调整
    # 确保角度在30到140度之间
    angle = max(30, min(140, angle))
    # 计算占空比
    print("angel",angle)
    s1.write_angle(angle)

    # 控制马达速度和方向
    if speed >= 0:
        p_1.value(1)
        p_2.value(0)
    else:
        p_1.value(0)
        p_2.value(1)
        speed = -speed

    # 将速度值映射到0-1023
    pwm9.write_analog(speed)

#===============================================================================================
    
m = 300  # 速度调整系数

def calculate_follow_and_turn(distance, x, width, center_radius, stop_h, stop_l, m):
    """
    计算物体的跟随和转向速度。

    参数:
    - distance: 物体到摄像头的距离。
    - x: 物体在摄像头视野中的水平位置。
    - width: 摄像头视野的宽度。
    - center_radius: 中心圈半径，用于确定物体是否偏离中心。
    - stop_h: 停止距离的上限。
    - stop_l: 停止距离的下限。
    - m: 速度调整的系数。

    返回:
    - speed: 计算出的速度（整数）。
    - direction: 计算出的转向角度（整数）。
    """
    # 定义最大速度和最大转向角度
    max_speed = 1000
    max_turn = 30

    # 计算物体相对于中心的偏移量
    center_x = width / 2
    offset = x - center_x

    # 根据距离调整速度
    if stop_l <= distance <= stop_h:
        speed = 0  # 停止
    elif distance < stop_l:
        speed = -round((stop_l - distance) * max_speed / m)  # 后退，距离越小速度越快
    elif distance > stop_h:
        speed = round((distance - stop_h) * max_speed / m)  # 前进，距离越大速度越快

    # 限制速度不超过最大速度
    speed = max(-max_speed, min(speed, max_speed))

    # 转换速度为整数
    speed = int(speed)


    # 根据偏移量调整转向角度
    if offset > center_radius:
        direction = round(max_turn * ((offset - center_radius) / (width / 2 - center_radius)))
    elif offset < -center_radius:
        direction = round(max_turn * ((offset + center_radius) / (width / 2 - center_radius)))
    else:
        direction = 0

    # 转换方向为整数
    direction = int(direction)

    return speed, direction




while True:
    x, y, w, h=(0,0,0,0)

    ret, img = vd.read()
     # 进行左右对称
    #img = cv2.resize(img, (240, 320)) # 调整图像尺寸与行空板相同
    img = cv2.flip(img, 1)
    #img = cv2.rotate(img,cv2.ROTATE_90_CLOCKWISE) #旋转90度

    # 将图像转换为灰度图，因为人脸检测通常在灰度图上进行
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

    # 检测图像中的人脸
    faces = face_cascade.detectMultiScale(gray, scaleFactor=1.1, minNeighbors=5, minSize=(30, 30), flags=cv2.CASCADE_SCALE_IMAGE)

    # 在检测到的人脸周围画矩形框
    for (x, y, w, h) in faces:
        cv2.rectangle(img, (x, y), (x+w, y+h), (255, 0, 0), 2)

    x = x + w / 2

    # 得到像素距离转为实际cm距离的公式 y = Ax^2 + Bx + C
    distanceCM =int(A*w**2 + B*w + C)
    #print('distance CM:', distanceCM)


    img= Image.fromarray(cv2.cvtColor(img, cv2.COLOR_BGR2RGB)) #将opecv图片格式转化为unihiker库图片格式
    bgImg.config(image=img)
    

    ltTxt1.config(text=(str(distanceCM)))
    ltTxt2.config(text=(str(w)))

    print("distance:",distanceCM,"X:",x)

    speed, direction = calculate_follow_and_turn(distanceCM, x, width, center_radius, stop_h, stop_l, m)
    print("speed:",distanceCM,"direction:",direction)
    if distanceCM!=945:
        car(speed,direction)
    


    
